@@ -15,6 +15,7 @@
 #include <KSharedConfig>
 
 #include <QDebug>
+#include <QQmlEngine>
 
 const QString TZ_CFG_GROUP = QStringLiteral("Timezones");
 
@@ -39,26 +40,55 @@ void AddLocationModel::load()
     beginResetModel();
 
     m_list.clear();
+    m_addedLocations.clear();
 
     auto config = KSharedConfig::openConfig();
     KConfigGroup timezoneGroup = config->group(TZ_CFG_GROUP);
 
-    // add other configured time zones
-    for (QByteArray id : QTimeZone::availableTimeZoneIds()) {
-        bool shown = timezoneGroup.readEntry(id.data(), false);
+    const auto timeZoneIds = QTimeZone::availableTimeZoneIds();
+    m_list.reserve(timeZoneIds.count());
+    for (const QByteArray &id : timeZoneIds) {
         m_list.append(QTimeZone(id));
 
+        const bool shown = timezoneGroup.readEntry(id.constData(), false);
         if (shown) {
-            m_addedLocations.insert(QString::fromStdString(id.toStdString()));
+            m_addedLocations.insert(id);
         }
     }
 
     endResetModel();
 }
 
+void AddLocationModel::updateAddedLocations()
+{
+    const KConfigGroup timezoneGroup = KSharedConfig::openConfig()->group(TZ_CFG_GROUP);
+    for (int i = 0; i < m_list.count(); ++i) {
+        const QTimeZone &timeZone = m_list.at(i);
+        const QByteArray timeZoneId = timeZone.id();
+
+        const bool wasShown = m_addedLocations.contains(timeZoneId);
+        const bool shown = timezoneGroup.readEntry(timeZoneId.constData(), false);
+
+        if (shown == wasShown) {
+            continue;
+        }
+
+        if (shown && !wasShown) {
+            m_addedLocations.insert(timeZoneId);
+        } else if (!shown && wasShown) {
+            m_addedLocations.remove(timeZoneId);
+        }
+
+        const QModelIndex changedIndex = index(i, 0);
+        Q_EMIT dataChanged(changedIndex, changedIndex, {AddLocationModel::AddedRole});
+    }
+}
+
 int AddLocationModel::rowCount(const QModelIndex &parent) const
 {
-    Q_UNUSED(parent);
+    if (parent.isValid()) {
+        return 0;
+    }
     return m_list.count();
 }
 
@@ -67,13 +97,13 @@ QVariant AddLocationModel::data(const QModelIndex &index, int role) const
     if (!index.isValid())
         return QVariant();
 
-    auto tz = m_list[index.row()];
-    QString id = QString::fromStdString(tz.id().replace("_", " ").toStdString());
+    const QTimeZone &tz = m_list.at(index.row());
+    const QByteArray id = tz.id().replace("_", " ");
 
     switch (role) {
     case CityRole: {
-        auto split = id.split(QLatin1Char('/'));
-        return split[split.size() - 1];
+        const int lastSlashIdx = id.lastIndexOf('/');
+        return id.mid(lastSlashIdx + 1);
     }
     case CountryRole:
         return QLocale::territoryToString(tz.territory());
@@ -82,11 +112,11 @@ QVariant AddLocationModel::data(const QModelIndex &index, int role) const
     case IdRole:
         return id;
     case AddedRole:
-        return m_addedLocations.find(id) != m_addedLocations.end();
+        return m_addedLocations.contains(id);
     case CurrentTimeRole: {
         QDateTime time = QDateTime::currentDateTime();
         time = time.toTimeZone(tz);
-        return KclockFormat().formatTimeString(time.time().hour(), time.time().minute());
+        return KclockFormat::instance()->formatTimeString(time.time().hour(), time.time().minute());
     }
     }
     return QVariant();
@@ -109,27 +139,69 @@ AddLocationSearchModel *AddLocationSearchModel::instance()
     return singleton;
 }
 
+AddLocationSearchModel *AddLocationSearchModel::create(QQmlEngine *qmlEngine, QJSEngine *jsEngine)
+{
+    Q_UNUSED(qmlEngine);
+    Q_UNUSED(jsEngine);
+    auto *model = instance();
+    QQmlEngine::setObjectOwnership(model, QQmlEngine::CppOwnership);
+    return model;
+}
+
 AddLocationSearchModel::AddLocationSearchModel(QObject *parent)
     : QSortFilterProxyModel(parent)
 {
     setFilterCaseSensitivity(Qt::CaseInsensitive);
     setSourceModel(AddLocationModel::instance());
-    setFilterRole(AddLocationModel::IdRole);
     sort(0);
 }
 
-void AddLocationSearchModel::addLocation(int index)
+QString AddLocationSearchModel::query() const
+{
+    return m_query;
+}
+
+void AddLocationSearchModel::setQuery(const QString &query)
+{
+    if (m_query == query) {
+        return;
+    }
+
+    m_query = query;
+    invalidateFilter();
+    Q_EMIT queryChanged(query);
+}
+
+void AddLocationSearchModel::addLocation(int row)
 {
     auto config = KSharedConfig::openConfig();
     KConfigGroup timezoneGroup = config->group(TZ_CFG_GROUP);
-    QString ianaId = data(this->index(index, 0), AddLocationModel::IdRole).toString().replace(QStringLiteral(" "), QStringLiteral("_"));
+    const QModelIndex locationIndex = index(row, 0);
+    const QString ianaId = locationIndex.data(AddLocationModel::IdRole).toString().replace(QStringLiteral(" "), QStringLiteral("_"));
     timezoneGroup.writeEntry(ianaId, true);
     config->sync();
 
-    AddLocationModel::instance()->load();
+    AddLocationModel::instance()->updateAddedLocations();
     SavedLocationsModel::instance()->load();
 }
 
+bool AddLocationSearchModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
+{
+    if (!m_query.isEmpty()) {
+        const QModelIndex sourceIndex = sourceModel()->index(source_row, 0, source_parent);
+
+        const QString timezoneId = sourceIndex.data(AddLocationModel::IdRole).toString();
+        const QString city = sourceIndex.data(AddLocationModel::CityRole).toString();
+        const QString country = sourceIndex.data(AddLocationModel::CountryRole).toString();
+        if (!timezoneId.contains(m_query, Qt::CaseInsensitive) && !city.contains(m_query, Qt::CaseInsensitive)
+            && !country.contains(m_query, Qt::CaseInsensitive)) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
 bool AddLocationSearchModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
 {
     return QVariant::compare(sourceModel()->data(left, AddLocationModel::CityRole), sourceModel()->data(right, AddLocationModel::CityRole))
