@@ -16,6 +16,11 @@
 #include <QTimeZone>
 #include <QTimer>
 
+#include <qqmlintegration.h>
+
+class QJSEngine;
+class QQmlEngine;
+
 class AddLocationModel : public QAbstractListModel
 {
     Q_OBJECT
@@ -41,23 +46,37 @@ public:
 
 public Q_SLOTS:
     void load();
+    void updateAddedLocations();
 
 private:
     QList<QTimeZone> m_list;
-    QSet<QString> m_addedLocations;
+    QSet<QByteArray> m_addedLocations;
 };
 
 class AddLocationSearchModel : public QSortFilterProxyModel
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
+
+    Q_PROPERTY(QString query READ query WRITE setQuery NOTIFY queryChanged)
 
 public:
     static AddLocationSearchModel *instance();
+    static AddLocationSearchModel *create(QQmlEngine *qmlEngine, QJSEngine *jsEngine);
 
-    explicit AddLocationSearchModel(QObject *parent = nullptr);
+    QString query() const;
+    void setQuery(const QString &query);
+    Q_SIGNAL void queryChanged(const QString &query);
 
-    Q_INVOKABLE void addLocation(int index);
+    Q_INVOKABLE void addLocation(int row);
 
 protected:
+    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override;
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const override;
+
+private:
+    explicit AddLocationSearchModel(QObject *parent = nullptr);
+
+    QString m_query;
 };
