@@ -14,6 +14,7 @@
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
+#include <QQmlEngine>
 #include <QtGlobal>
 
 TimerModel *TimerModel::instance()
@@ -22,6 +23,15 @@ TimerModel *TimerModel::instance()
     return singleton;
 }
 
+TimerModel *TimerModel::create(QQmlEngine *qmlEngine, QJSEngine *jsEngine)
+{
+    Q_UNUSED(qmlEngine);
+    Q_UNUSED(jsEngine);
+    auto *model = instance();
+    QQmlEngine::setObjectOwnership(model, QQmlEngine::CppOwnership);
+    return model;
+}
+
 TimerModel::TimerModel(QObject *parent)
     : QAbstractListModel{parent}
     , m_interface(new OrgKdeKclockTimerModelInterface(QStringLiteral("org.kde.kclockd"), QStringLiteral("/Timers"), QDBusConnection::sessionBus(), this))
@@ -30,6 +40,8 @@ TimerModel::TimerModel(QObject *parent)
         // connect timer signals
         connect(m_interface, SIGNAL(timerAdded(QString)), this, SLOT(addTimer(QString)));
         connect(m_interface, SIGNAL(timerRemoved(QString)), this, SLOT(removeTimer(QString)));
+        connect(m_interface, SIGNAL(defaultAudioLocationChanged()), this, SLOT(updateDefaultAudioLocation()));
+        updateDefaultAudioLocation();
     }
 
     // load timers
@@ -51,7 +63,9 @@ TimerModel::TimerModel(QObject *parent)
 
 int TimerModel::rowCount(const QModelIndex &parent) const
 {
-    Q_UNUSED(parent);
+    if (parent.isValid()) {
+        return 0;
+    }
     return m_timersList.size();
 }
 
@@ -70,31 +84,37 @@ QHash<int, QByteArray> TimerModel::roleNames() const
     return {{TimerRole, "timer"}};
 }
 
-void TimerModel::addNew(int length, QString label, QString commandTimeout)
+void TimerModel::addNew(int length, const QString &label, bool looping, const QString &commandTimeout)
 {
-    m_interface->addTimer(length, label, commandTimeout, false);
+    m_interface->addTimer(length, label, looping, commandTimeout, false);
 }
 
-void TimerModel::remove(int index)
+void TimerModel::remove(Timer *timer)
 {
-    if (index < 0 || index >= m_timersList.size())
-        return;
-
-    // request kclockd to remove timer, which will trigger a signal back to remove it from the UI
-    m_interface->removeTimer(m_timersList.at(index)->uuid().toString());
+    if (timer) {
+        // request kclockd to remove timer, which will trigger a signal back to remove it from the UI
+        m_interface->removeTimer(timer->uuid().toString());
+    }
 }
 
 void TimerModel::addTimer(QString uuid)
 {
     auto *timer = new Timer(uuid.remove(QRegularExpression(QStringLiteral("[{}-]"))));
+    connect(timer, &Timer::runningChanged, this, &TimerModel::runningTimerChanged);
 
+    Timer *oldRunningTimer = runningTimer();
     Q_EMIT beginInsertRows(QModelIndex(), 0, 0);
     m_timersList.insert(0, timer);
     Q_EMIT endInsertRows();
+    if (oldRunningTimer != runningTimer()) {
+        Q_EMIT runningTimerChanged();
+    }
 }
 
 void TimerModel::removeTimer(const QString &uuid)
 {
+    Timer *oldRunningTimer = runningTimer();
+
     for (int i = 0; i < m_timersList.size(); ++i) {
         if (m_timersList[i]->uuid().toString() == uuid) {
             beginRemoveRows(QModelIndex(), i, i);
@@ -103,6 +123,10 @@ void TimerModel::removeTimer(const QString &uuid)
             endRemoveRows();
         }
     }
+
+    if (oldRunningTimer != runningTimer()) {
+        Q_EMIT runningTimerChanged();
+    }
 }
 
 bool TimerModel::connectedToDaemon()
@@ -118,4 +142,39 @@ void TimerModel::setConnectedToDaemon(bool connectedToDaemon)
     }
 }
 
+QString TimerModel::defaultAudioLocation() const
+{
+    return m_defaultAudioLocation;
+}
+
+void TimerModel::setDefaultAudioLocation(const QString &location)
+{
+    m_interface->setProperty("defaultAudioLocation", location);
+}
+
+void TimerModel::updateDefaultAudioLocation()
+{
+    m_defaultAudioLocation = m_interface->defaultAudioLocation();
+    Q_EMIT defaultAudioLocationChanged();
+}
+
+Timer *TimerModel::runningTimer() const
+{
+    Timer *runningTimer = nullptr;
+    for (Timer *timer : std::as_const(m_timersList)) {
+        if (!timer->running()) {
+            continue;
+        }
+
+        if (!runningTimer) {
+            runningTimer = timer;
+        } else {
+            // More than one running timer, exit.
+            runningTimer = nullptr;
+            break;
+        }
+    }
+    return runningTimer;
+}
+
 #include "moc_timermodel.cpp"
